local job_helper = require 'jobs.job_helper'
local constants = require 'constants'

local BaseJob = class()

--[[
   A base class for all classes. Contains getters for job variables common to all classes
   TODO: Move functions out of classes that inherit from this class if its function is identical to the one here
]]

function BaseJob:initialize()
   job_helper.initialize(self._sv)
end

function BaseJob:create(entity)
   job_helper.initialize(self._sv, entity)
   self:restore()
end

function BaseJob:restore()
end

function BaseJob:promote(json, options)
   job_helper.promote(self._sv, json)

   self.__saved_variables:mark_changed()
end

-- Returns the level the character has in this class
function BaseJob:get_job_level()
   return self._sv.last_gained_lv
end

-- Returns whether we're at max level.
-- NOTE: If max level is nto declared, returns false
function BaseJob:is_max_level()
   return self._sv.is_max_level 
end

-- Returns all the data for all the levels
function BaseJob:get_level_data()
   return self._sv.level_data
end

-- Given the ID of a perk, find out if we have the perk. 
function BaseJob:has_perk(id)
   return self._sv.attained_perks[id]
end

function BaseJob:demote()
   self._sv.is_current_class = false
   self.__saved_variables:mark_changed()
end

return BaseJob
