local ClericClass = class()
local job_helper = require 'jobs.job_helper'
local BaseJob = require 'jobs.base_job'
radiant.mixin(ClericClass, BaseJob)

--- Public functions, required for all classes

function ClericClass:initialize()
   job_helper.initialize(self._sv)
end

function ClericClass:create(entity)
   job_helper.initialize(self._sv, entity)
   self:restore()
end

--Always do these things
function ClericClass:restore()
   --If we're loading and there is no entity anymore (b/c killed, etc) then don't progress
   --TODO: shouldn't we find some way to retire these controllers if they're no longer needed?
   if not self._sv._entity or not self._sv._entity:is_valid() then
      return
   end

   self._job_component = self._sv._entity:get_component('stonehearth:job')

   self.__saved_variables:mark_changed()
end

-- Call when it's time to promote someone to this class
function ClericClass:promote(json, options)
   job_helper.promote(self._sv, json)

   self.__saved_variables:mark_changed()
end

-- Returns the level the character has in this class
function ClericClass:get_job_level()
   return self._sv.last_gained_lv
end

-- Returns whether we're at max level.
-- NOTE: If max level is nto declared, always false
function ClericClass:is_max_level()
   return self._sv.is_max_level 
end

-- Returns all the data for all the levels
function ClericClass:get_level_data()
   return self._sv.level_data
end

-- We keep an index of perks we've unlocked for easy lookup
function ClericClass:unlock_perk(id)
   self._sv.attained_perks[id] = true
   self.__saved_variables:mark_changed()
end

-- Given the ID of a perk, find out if we have the perk. 
function ClericClass:has_perk(id)
   return self._sv.attained_perks[id]
end

-- Call when it's time to level up in this class
function ClericClass:level_up()
   job_helper.level_up(self._sv)
   self.__saved_variables:mark_changed()
end

-- Call when it's time to demote someone from this class
function ClericClass:demote()
   self._sv.is_current_class = false
   self.__saved_variables:mark_changed()
end

--- Functions specific to level up
function ClericClass:apply_chained_buff(args)
   radiant.entities.remove_buff(self._sv._entity, args.last_buff)
   radiant.entities.add_buff(self._sv._entity, args.buff_name)   
end

function ClericClass:apply_buff(args)
   radiant.entities.add_buff(self._sv._entity, args.buff_name)   
end

function ClericClass:remove_buff(args)
   radiant.entities.remove_buff(self._sv._entity, args.buff_name)
end

--Add or remove a type of combat action the footman can choose in melee
function ClericClass:add_combat_action(args)
   job_helper.add_equipment(self._sv, args)
   local combat_state = stonehearth.combat:get_combat_state(self._sv._entity)
   combat_state:recompile_combat_actions(args.action_type)
end

function ClericClass:remove_combat_action(args)
   job_helper.remove_equipment(self._sv, args)
   local combat_state = stonehearth.combat:get_combat_state(self._sv._entity)
   combat_state:recompile_combat_actions(args.action_type)
end

return ClericClass