local CraftingJob = require 'jobs.crafting_job'

local HerbalistClass = class()
radiant.mixin(HerbalistClass, CraftingJob)

function HerbalistClass:initialize()
   CraftingJob.initialize(self)
   self._sv.max_num_attended_hearthlings = 2
end

function HerbalistClass:create(entity)
   CraftingJob.create(self, entity)
end

function HerbalistClass:restore()
   CraftingJob.restore(self)
   if self._sv.is_current_class then
      --self:_create_xp_listeners()
      self:_register_with_town()
   end
end

function HerbalistClass:promote(json, options)
   CraftingJob.promote(self, json, options)
   self._sv.max_num_attended_hearthlings = json.initial_num_attended_hearthlings or 2
   if self._sv.max_num_attended_hearthlings > 0 then
      self:_register_with_town()
   end
   self.__saved_variables:mark_changed()
end

function HerbalistClass:increase_attended_hearthlings(args)
   self._sv.max_num_attended_hearthlings = args.max_num_attended_hearthlings
   self:_register_with_town() -- re-register with the town because number of max attended hearthlings is increased
   self.__saved_variables:mark_changed()
end

-- Registers the medic with the town
function HerbalistClass:_register_with_town()
   local player_id = radiant.entities.get_player_id(self._sv._entity)
   local town = stonehearth.town:get_town(player_id)
   if town then
      town:add_medic(self._sv._entity, self._sv.max_num_attended_hearthlings)
   end
end

function HerbalistClass:_create_xp_listeners()
   CraftingJob._create_xp_listeners(self)
   self._on_heal_entity_listener = radiant.events.listen(self._sv._entity, 'stonehearth:healer:healed_entity', self, self._on_healed_entity)
end

function HerbalistClass:_remove_xp_listeners()
   CraftingJob._remove_xp_listeners(self)
   if self._on_heal_entity_listener then
      self._on_heal_entity_listener:destroy()
      self._on_heal_entity_listener = nil
   end
end

function HerbalistClass:_on_healed_entity(args)
   local exp = self._sv.xp_rewards['heal_entity']
   if exp then
      self._job_component:add_exp(exp)
   end
end

-- Call when it's time to demote
function HerbalistClass:demote()
   
   local player_id = radiant.entities.get_player_id(self._sv._entity)
   local town = stonehearth.town:get_town(player_id)
   if town then
      town:remove_medic(self._sv._entity)
   end

   CraftingJob.demote(self)
end

return HerbalistClass
