local job_helper = {}

--Make initializtion common to all classes
function job_helper.initialize(sv, entity)
   sv._entity = entity
   sv.job_name = nil
   sv.last_gained_lv = 0
   sv.is_current_class = false
   sv.max_level = 0
   sv.is_max_level = false
   sv.attained_perks = {}
   sv.parent_level_requirement = 0
   sv.xp_rewards = nil
   sv.level_data = nil
   sv.is_combat_class = nil
   sv.equipment = {}
end

-- Save promotion-related variables common to all classes
function job_helper.promote(sv, json)
   sv.is_current_class = true
   sv.job_name = json.display_name
   sv.max_level = json.max_level

   if not sv.is_max_level then
      sv.is_max_level = false
   end

   if json.parent_level_requirement then
      sv.parent_level_requirement = json.parent_level_requirement
   end

   if json.xp_rewards then
      sv.xp_rewards = json.xp_rewards
   end

   if json.level_data then
      sv.level_data = json.level_data
   end
   
   local is_combat_class = string.find(json.roles, "combat")

   -- If being promoted to a combat class, make sure entity
   -- is removed from militia
   if is_combat_class then
      local population = stonehearth.population:get_population(sv._entity)
      population:update_militia_command({}, {}, sv._entity:get_id(), false)
   end

   --Unless you are a combat class (role is combat) remove self
   --from any preexisting parties on promote
   --clear self of any combat commands
   if not is_combat_class then
      sv.is_combat_class = false
      local curr_party = stonehearth.unit_control:get_party_for_entity_command({}, {}, sv._entity)
      if curr_party then
         local party_component = curr_party:get_component('stonehearth:party')
         if party_component then
            party_component:remove_member(sv._entity:get_id())
         end
      end
      stonehearth.combat_server_commands:clear_entity_of_combat_commands(sv._entity:get_id())
   else
      sv.is_combat_class = true
   end

end

-- Do level up logic common to all classes
function job_helper.level_up(sv)
   sv.last_gained_lv = sv.last_gained_lv + 1

   if sv.last_gained_lv == sv.max_level then
      sv.is_max_level = true
   end
end

-- Add equipment to the character, and also to a tracking table for removal purposes later
function job_helper.add_equipment(sv, args)
   sv.equipment[args.equipment] = radiant.entities.create_entity(args.equipment)
   radiant.entities.equip_item(sv._entity, sv.equipment[args.equipment])
end

-- Remove equipment from the character
function job_helper.remove_equipment(sv, args)
   if sv.equipment[args.equipment] then
      radiant.entities.unequip_item(sv._entity, sv.equipment[args.equipment])
      sv.equipment[args.equipment] = nil
   end
end

return job_helper